#include "BezierHelper.h"

namespace sofa {

namespace bezier4D {


Point3 cross(Point3 u, Point3 v)  {
    Point3 r;
    r[0] = u[1]*v[2] - u[2]*v[1];
    r[1] = u[2]*v[0] - u[0]*v[2];
    r[2] = u[0]*v[1] - u[1]*v[0];
    return r;
}

double dot(Point3 u, Point3 v) {
    return u[0] * v[0] + u[1] * v[1] + u[2] * v[2];
}

/*void RigidCurvePoint4::draw(DrawInfoMap& dim) {
    DrawInfo di = dim.getInfo("mappingPointFrames");
    if (di.show) {
        qglviewer::Vec p1, p2;

        p1 = getGLPos();
        p2 = getGLPosAddVec(TV, di.size);
        glColor3d(0.6,0.0,0.0);
        QGLViewer::drawArrow(p1, p2);

        p2 = getGLPosAddVec(NV, di.size);
        glColor3d(0.0,0.6,0.0);
        QGLViewer::drawArrow(p1, p2);

        p2 = getGLPosAddVec(BV, di.size);
        glColor3d(0.0,0.0,0.6);
        QGLViewer::drawArrow(p1, p2);
    }

    di = dim.getInfo("mappingPointTargets");
    if (di.show) {
        if (hasMatchingPosition) {
            qglviewer::Vec p1, p2;
            p1 = getGLPos();
            p2 = getGLMatchingPos();

            glColor4fv(di.color.p());
            glLineWidth(di.size);
            QGLViewer::drawArrow(p1, p2, di.size, 6);
        }
    }
}*/

/*bool RigidCurvePoint4::mapPointToTetraVertices(ChMesh* pMesh, size_t el, Point4& bc) {
    //double df = 0.0;
    //for (size_t i = 0; i < 3; i++)
    //    df += SQR(pt[i] - pMesh->centers[el][i]);
    //df = sqrt(df);
    //if (df < 0.01) {
    //    cout << df << endl;
    //    cout << el << endl;
    //}

    vector<vector<double> >& vertices = pMesh->vecElementCoords[el];
    double **invVertMat = pMesh->invVertMatrices[el];

    bool accept = true;
    //cout << "BC1= " << bc << endl;
    for (size_t i = 0; i < 3; i++)
        for (size_t j = 0; j < 3; j++)
            bc[i] += invVertMat[i][j]*(at(j) - vertices[3][j]);

    bc[3] = 1 - bc[0] - bc[1] - bc[2];

    double margin = 0.0;
    //double maxDiff = 0.0;
    for (size_t i = 0; i < 4; i++)
        accept = accept && (margin <= bc[i] && bc[i] <= 1.0-margin);

    //if (circleRatioUpLimit != 0.0) {
    //    double dst =  sqr(pMesh->innerCircles[el]);
    //    accept = accept && (dst*circleRatioLowLimit <= diff2 && diff2 < dst*circleRatioUpLimit);
    //}

    if (accept)  {        
        if (pMesh->mappedElements.find(el) == pMesh->mappedElements.end()) {
            mappedTetra = el;
            baryCoord = bc;
            pMesh->mappedElements.insert(el);            
            //cout << "Point [" << vesselID << "][" << segmentID << "][" << pi << "] mapped to " << el << " " <<  bc << endl;
        } else {
            accept = false;
        }
    }
    return accept;
}*/

Point3 getDiff4(Point4 a, Point4 b) {
    Point3 r;
    for (size_t i = 0; i < 3; i++)
        r[i] = b[i] - a[i];
    return(r);
}


double B0(double u)
{
    double tmp = 1.0 - u;
    return (tmp * tmp * tmp);
}
double B1(double u)
{
    double tmp = 1.0 - u;
    return (3 * u * (tmp * tmp));
}
double B2(double u)
{
    double tmp = 1.0 - u;
    return (3 * u * u * tmp);
}
double B3(double u)
{
    return (u * u * u);
}

Vector4 ComputeLeftTangent(Point4 p1, Point4 p2)
{
    Vector4	tHat1;
    tHat1 = V4Sub(p2, p1);
    V4Normalize(tHat1);
    return tHat1;
}

Vector4 ComputeRightTangent(Point4 p1, Point4 p2)
{
    Vector4	tHat2;
    tHat2 = V4Sub(p1,p2);
    V4Normalize(tHat2);
    return tHat2;
}

Vector4 ComputeCenterTangent(Point4 p1, Point4 p2, Point4 p3) {
    Vector4	V1, V2, tHatCenter;

    V1 = V4Sub(p1, p2);
    V2 = V4Sub(p2, p3);
    tHatCenter[0] = (V1[0] + V2[0])/2.0;
    tHatCenter[1] = (V1[1] + V2[1])/2.0;
    tHatCenter[2] = (V1[2] + V2[2])/2.0;
    tHatCenter[3] = (V1[3] + V2[3])/2.0;
    V4Normalize(tHatCenter);
    return tHatCenter;
}

double V4SquaredLength(Vector4 a)
{
    return((a[0] * a[0])+(a[1] * a[1])+(a[2] * a[2])+(a[3] * a[3]));
}

double V4Length(Vector4 a)
{
    return(sqrt(V4SquaredLength(a)));
}

Vector4 V4Negated(Vector4 v)
{
    Vector4 r;
    r[0] = -v[0]; r[1] = -v[1];  r[2] = -v[2]; r[3] = -v[3];
    return(r);
}
void V4Negate(Vector4& v) {
    for (size_t i = 0; i < 4; i++)
        v[i] = -v[i];
}

void V4Normalize(Vector4& v)
{   
    double len = V4Length(v);
    if (len != 0.0) { v[0] /= len;  v[1] /= len; v[2] /= len; v[3] /= len;}
}

Vector4 V4Normalized(Vector4 v)
{
    Vector4 r;
    double len = V4Length(v);
    if (len != 0.0) { r[0] = v[0]/len;  r[1] = v[1]/len; r[2]= v[2]/len; r[3] = v[3]/len;}
    return r;
}

void V4ImposeLength(Vector4& v,double newlen)
{    
    double len = V4Length(v);
    if (len != 0.0)
        for (size_t i = 0; i < 4; i++)
            v[i] *= newlen/len;
    else
        std::cerr << "Imposing length to zero vector!" << std::endl;
}

void V4Scale3(Vector4& v, double factor) {
    for (size_t i = 0; i < 3; i++)
        v[i] *= factor;
}

Vector4 V4Scaled3(Vector4& v, double factor) {
    Vector4 r;
    for (size_t i = 0; i < 3; i++)
        r[i] = v[i] * factor;
    r[3] = v[3];
    return(r);
}



Vector4 V4ImposedLength(Vector4 v,double newlen)
{
    double len = V4Length(v);
    Vector4 r;
    if (len != 0.0)
        for (size_t i = 0; i < 4; i++)
            r[i] = v[i] * newlen/len;
    else
        std::cerr << "Imposing length to zero vector!" << std::endl;
    return(r);
}

Vector4 V4Add(Vector4 a,Vector4 b)
{
    Vector4 c;
    c[0] = a[0]+b[0];  c[1] = a[1]+b[1];  c[2] = a[2]+b[2]; c[3] = a[3]+b[3];
    return(c);
}

Vector4 V4Sub(Vector4 a,Vector4 b)
{
    Vector4 c;
    c[0] = a[0]-b[0];  c[1] = a[1]-b[1];  c[2] = a[2]-b[2]; c[3] = a[3]-b[3];
    return(c);
}

double V4Dot(Vector4 a,Vector4 b)
{
    return((a[0]*b[0])+(a[1]*b[1])+(a[2]*b[2])+(a[3]*b[3]));
}

double V4Dist(Point4 a,Point4 b)
{
    double dx = a[0] - b[0];
    double dy = a[1] - b[1];
    double dz = a[2] - b[2];
    double dw = a[3] - b[3];
    return(sqrt((dx*dx)+(dy*dy)+(dz*dz)+(dw*dw)));
}

double V4Dist3(Point4 a,Point4 b)
{
    double dx = a[0] - b[0];
    double dy = a[1] - b[1];
    double dz = a[2] - b[2];
    return(sqrt((dx*dx)+(dy*dy)+(dz*dz)));
}

Point4 BezierII(int degree, vector<Point4>&V, double t)
{
    int 	i, j;
    Point4 	Q;	        /* Point on curve at parameter t	*/
    vector<Point4> Vtemp = V;		/* Local copy of control points		*/

    /* Triangle computation	*/
    for (i = 1; i <= degree; i++) {
        for (j = 0; j <= degree-i; j++) {
            Vtemp[j][0] = (1.0 - t) * Vtemp[j][0] + t * Vtemp[j+1][0];
            Vtemp[j][1] = (1.0 - t) * Vtemp[j][1] + t * Vtemp[j+1][1];
            Vtemp[j][2] = (1.0 - t) * Vtemp[j][2] + t * Vtemp[j+1][2];
            Vtemp[j][3] = (1.0 - t) * Vtemp[j][3] + t * Vtemp[j+1][3];
        }
    }

    Q = Vtemp[0];

    return Q;
}

Point4 frame2Quater(Point3 vT, Point3 vB, Point3 vN)
{
    double m[3][3];

    for (size_t i=0 ; i<3 ;i++)
    {
        m[i][0] = vT[i];
        m[i][1] = vB[i];
        m[i][2] = vN[i];
    }

    Point4 quat;
    double tr, s;

    tr = (double)(m[0][0] + m[1][1] + m[2][2]);

    // check the diagonal
    if (tr > 0)
    {
        s = (float)sqrt (tr + 1);
        quat[3] = s * 0.5f; // w OK
        s = 0.5f / s;
        quat[0] = (double)((m[2][1] - m[1][2]) * s); // x OK
        quat[1] = (double)((m[0][2] - m[2][0]) * s); // y OK
        quat[2] = (double)((m[1][0] - m[0][1]) * s); // z OK
    }
    else
    {
        if (m[1][1] > m[0][0] && m[2][2] <= m[1][1])
        {
            s = (double)sqrt ((m[1][1] - (m[2][2] + m[0][0])) + 1.0f);

            quat[1] = s * 0.5f; // y OK

            if (s != 0.0f)
                s = 0.5f / s;

            quat[2] = (double)((m[1][2] + m[2][1]) * s); // z OK
            quat[0] = (double)((m[0][1] + m[1][0]) * s); // x OK
            quat[3] = (double)((m[0][2] - m[2][0]) * s); // w OK
        }
        else if ((m[1][1] <= m[0][0]  &&  m[2][2] > m[0][0])  ||  (m[2][2] > m[1][1]))
        {
            s = (double)sqrt ((m[2][2] - (m[0][0] + m[1][1])) + 1.0f);

            quat[2] = s * 0.5f; // z OK

            if (s != 0.0f)
                s = 0.5f / s;

            quat[0] = (double)((m[2][0] + m[0][2]) * s); // x OK
            quat[1] = (double)((m[1][2] + m[2][1]) * s); // y OK
            quat[3] = (double)((m[1][0] - m[0][1]) * s); // w OK
        }
        else
        {
            s = (double)sqrt ((m[0][0] - (m[1][1] + m[2][2])) + 1.0f);

            quat[0] = s * 0.5f; // x OK

            if (s != 0.0f)
                s = 0.5f / s;

            quat[1] = (double)((m[0][1] + m[1][0]) * s); // y OK
            quat[2] = (double)((m[2][0] + m[0][2]) * s); // z OK
            quat[3] = (double)((m[2][1] - m[1][2]) * s); // w OK
        }
    }

    return(quat);
}

}

}

